# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: grpc_server.proto
# Protobuf Python Version: 5.28.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    1,
    '',
    'grpc_server.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11grpc_server.proto\x12\x05\x61gent\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1bgoogle/protobuf/empty.proto\".\n\x0fGeneralResponse\x12\n\n\x02ok\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"\xe9\x01\n\tLLMConfig\x12\x18\n\x0b\x63onfig_name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x06\x63lient\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x12\n\x05model\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x17\n\nmax_tokens\x18\x04 \x01(\x05H\x03\x88\x01\x01\x12\x18\n\x0btemperature\x18\x05 \x01(\x02H\x04\x88\x01\x01\x12\x15\n\x08\x61pi_base\x18\x06 \x01(\tH\x05\x88\x01\x01\x42\x0e\n\x0c_config_nameB\t\n\x07_clientB\x08\n\x06_modelB\r\n\x0b_max_tokensB\x0e\n\x0c_temperatureB\x0b\n\t_api_base\"@\n\nNodeServer\x12\x13\n\x0bserver_type\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\x12\x0f\n\x07node_id\x18\x03 \x01(\t\"m\n\x0fNodeConfigInput\x12\n\n\x02ip\x18\x01 \x01(\t\x12\x16\n\thttp_port\x18\x02 \x01(\x05H\x00\x88\x01\x01\x12\x18\n\x0bserver_type\x18\x03 \x01(\tH\x01\x88\x01\x01\x42\x0c\n\n_http_portB\x0e\n\x0c_server_type\"\xc7\x03\n\nNodeConfig\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05owner\x18\x02 \x01(\t\x12\x12\n\npublic_key\x18\x03 \x01(\t\x12\n\n\x02ip\x18\x04 \x01(\t\x12\x13\n\x0bserver_type\x18\x05 \x01(\t\x12\x11\n\thttp_port\x18\x06 \x01(\x05\x12\x13\n\x0bnum_servers\x18\x07 \x01(\x05\x12\"\n\x07servers\x18\x08 \x03(\x0b\x32\x11.agent.NodeServer\x12\x15\n\rollama_models\x18\t \x03(\t\x12\x13\n\x0b\x64ocker_jobs\x18\n \x01(\x08\x12\r\n\x05ports\x18\x0b \x03(\x05\x12\x19\n\x0crouting_type\x18\x0c \x01(\tH\x00\x88\x01\x01\x12\x18\n\x0brouting_url\x18\r \x01(\tH\x01\x88\x01\x01\x12\x15\n\x08num_gpus\x18\x0e \x01(\x05H\x02\x88\x01\x01\x12\x11\n\x04\x61rch\x18\x0f \x01(\tH\x03\x88\x01\x01\x12\x0f\n\x02os\x18\x10 \x01(\tH\x04\x88\x01\x01\x12\x10\n\x03ram\x18\x11 \x01(\x03H\x05\x88\x01\x01\x12\x11\n\x04vram\x18\x12 \x01(\x03H\x06\x88\x01\x01\x42\x0f\n\r_routing_typeB\x0e\n\x0c_routing_urlB\x0b\n\t_num_gpusB\x07\n\x05_archB\x05\n\x03_osB\x06\n\x04_ramB\x07\n\x05_vram\"\x9b\x02\n\x06Module\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x0e\n\x06\x61uthor\x18\x04 \x01(\t\x12\x12\n\nmodule_url\x18\x05 \x01(\t\x12\x18\n\x0bmodule_type\x18\x06 \x01(\tH\x00\x88\x01\x01\x12\x1b\n\x0emodule_version\x18\x07 \x01(\tH\x01\x88\x01\x01\x12\x1e\n\x11module_entrypoint\x18\x08 \x01(\tH\x02\x88\x01\x01\x12\x1b\n\x0e\x65xecution_type\x18\t \x01(\tH\x03\x88\x01\x01\x42\x0e\n\x0c_module_typeB\x11\n\x0f_module_versionB\x14\n\x12_module_entrypointB\x11\n\x0f_execution_type\"\xd0\x02\n\x14\x44\x61taGenerationConfig\x12\x19\n\x0csave_outputs\x18\x01 \x01(\x08H\x00\x88\x01\x01\x12\"\n\x15save_outputs_location\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x1e\n\x11save_outputs_path\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x18\n\x0bsave_inputs\x18\x04 \x01(\x08H\x03\x88\x01\x01\x12!\n\x14save_inputs_location\x18\x05 \x01(\tH\x04\x88\x01\x01\x12\x1d\n\x10\x64\x65\x66\x61ult_filename\x18\x06 \x01(\tH\x05\x88\x01\x01\x42\x0f\n\r_save_outputsB\x18\n\x16_save_outputs_locationB\x14\n\x12_save_outputs_pathB\x0e\n\x0c_save_inputsB\x17\n\x15_save_inputs_locationB\x13\n\x11_default_filename\"\xdb\x01\n\x0e\x42\x61seDeployment\x12(\n\x0bnode_config\x18\x01 \x01(\x0b\x32\x11.agent.NodeConfigH\x00\x12,\n\nnode_input\x18\x02 \x01(\x0b\x32\x16.agent.NodeConfigInputH\x00\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x1d\n\x06module\x18\x04 \x01(\x0b\x32\r.agent.Module\x12\'\n\x06\x63onfig\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x13\n\x0binitialized\x18\x06 \x01(\x08\x42\x06\n\x04node\"\xb8\x02\n\x0eToolDeployment\x12(\n\x0bnode_config\x18\x01 \x01(\x0b\x32\x11.agent.NodeConfigH\x00\x12,\n\nnode_input\x18\x02 \x01(\x0b\x32\x16.agent.NodeConfigInputH\x00\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x1d\n\x06module\x18\x04 \x01(\x0b\x32\r.agent.Module\x12\'\n\x06\x63onfig\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct\x12@\n\x16\x64\x61ta_generation_config\x18\x06 \x01(\x0b\x32\x1b.agent.DataGenerationConfigH\x01\x88\x01\x01\x12\x13\n\x0binitialized\x18\x07 \x01(\x08\x42\x06\n\x04nodeB\x19\n\x17_data_generation_config\"\xd1\x03\n\x0f\x41gentDeployment\x12(\n\x0bnode_config\x18\x01 \x01(\x0b\x32\x11.agent.NodeConfigH\x00\x12,\n\nnode_input\x18\x02 \x01(\x0b\x32\x16.agent.NodeConfigInputH\x00\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x1d\n\x06module\x18\x04 \x01(\x0b\x32\r.agent.Module\x12\'\n\x06\x63onfig\x18\x05 \x01(\x0b\x32\x17.google.protobuf.Struct\x12@\n\x16\x64\x61ta_generation_config\x18\x06 \x01(\x0b\x32\x1b.agent.DataGenerationConfigH\x01\x88\x01\x01\x12/\n\x10tool_deployments\x18\x07 \x03(\x0b\x32\x15.agent.ToolDeployment\x12-\n\x0ekb_deployments\x18\x08 \x03(\x0b\x32\x15.agent.BaseDeployment\x12\x36\n\x17\x65nvironment_deployments\x18\t \x03(\x0b\x32\x15.agent.BaseDeployment\x12\x13\n\x0binitialized\x18\n \x01(\x08\x42\x06\n\x04nodeB\x19\n\x17_data_generation_config\"\xa3\x03\n\x10ModuleRunRequest\x12\x13\n\x0bmodule_type\x18\x01 \x01(\t\x12\x13\n\x0b\x63onsumer_id\x18\x02 \x01(\t\x12\'\n\x06inputs\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x32\n\x10\x61gent_deployment\x18\x04 \x01(\x0b\x32\x16.agent.AgentDeploymentH\x00\x12\x30\n\x0ftool_deployment\x18\x05 \x01(\x0b\x32\x15.agent.ToolDeploymentH\x00\x12\x32\n\x11memory_deployment\x18\x06 \x01(\x0b\x32\x15.agent.BaseDeploymentH\x00\x12.\n\rkb_deployment\x18\x07 \x01(\x0b\x32\x15.agent.BaseDeploymentH\x00\x12\x37\n\x16\x65nvironment_deployment\x18\x08 \x01(\x0b\x32\x15.agent.BaseDeploymentH\x00\x12+\n\x11orchestrator_runs\x18\t \x03(\x0b\x32\x10.agent.ModuleRunB\x0c\n\ndeployment\"\x90\x06\n\tModuleRun\x12\x13\n\x0bmodule_type\x18\x01 \x01(\t\x12\x13\n\x0b\x63onsumer_id\x18\x02 \x01(\t\x12\'\n\x06inputs\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x32\n\x10\x61gent_deployment\x18\x04 \x01(\x0b\x32\x16.agent.AgentDeploymentH\x00\x12\x30\n\x0ftool_deployment\x18\x05 \x01(\x0b\x32\x15.agent.ToolDeploymentH\x00\x12\x32\n\x11memory_deployment\x18\x06 \x01(\x0b\x32\x15.agent.BaseDeploymentH\x00\x12.\n\rkb_deployment\x18\x07 \x01(\x0b\x32\x15.agent.BaseDeploymentH\x00\x12\x37\n\x16\x65nvironment_deployment\x18\x08 \x01(\x0b\x32\x15.agent.BaseDeploymentH\x00\x12+\n\x11orchestrator_runs\x18\t \x03(\x0b\x32\x10.agent.ModuleRun\x12\x0e\n\x06status\x18\n \x01(\t\x12\r\n\x05\x65rror\x18\x0b \x01(\x08\x12\x0f\n\x02id\x18\x0c \x01(\tH\x01\x88\x01\x01\x12\x0f\n\x07results\x18\r \x03(\t\x12\x1a\n\rerror_message\x18\x0e \x01(\tH\x02\x88\x01\x01\x12\x19\n\x0c\x63reated_time\x18\x0f \x01(\tH\x03\x88\x01\x01\x12\"\n\x15start_processing_time\x18\x10 \x01(\tH\x04\x88\x01\x01\x12\x1b\n\x0e\x63ompleted_time\x18\x11 \x01(\tH\x05\x88\x01\x01\x12\x15\n\x08\x64uration\x18\x12 \x01(\x01H\x06\x88\x01\x01\x12#\n\x16input_schema_ipfs_hash\x18\x13 \x01(\tH\x07\x88\x01\x01\x42\x0c\n\ndeploymentB\x05\n\x03_idB\x10\n\x0e_error_messageB\x0f\n\r_created_timeB\x18\n\x16_start_processing_timeB\x11\n\x0f_completed_timeB\x0b\n\t_durationB\x19\n\x17_input_schema_ipfs_hash\"5\n\x0eModuleRunCheck\x12\x13\n\x0bmodule_type\x18\x01 \x01(\t\x12\x0e\n\x06run_id\x18\x02 \x01(\t\"H\n\x10\x43heckUserRequest\x12\x14\n\x07user_id\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x12\n\npublic_key\x18\x02 \x01(\tB\n\n\x08_user_id\"J\n\x11\x43heckUserResponse\x12\x15\n\ris_registered\x18\x01 \x01(\x08\x12\n\n\x02id\x18\x02 \x01(\t\x12\x12\n\npublic_key\x18\x03 \x01(\t\")\n\x13RegisterUserRequest\x12\x12\n\npublic_key\x18\x01 \x01(\t\"6\n\x14RegisterUserResponse\x12\n\n\x02id\x18\x01 \x01(\t\x12\x12\n\npublic_key\x18\x02 \x01(\t2\x8a\x03\n\nGrpcServer\x12<\n\x08is_alive\x12\x16.google.protobuf.Empty\x1a\x16.agent.GeneralResponse\"\x00\x12\x38\n\x04stop\x12\x16.google.protobuf.Empty\x1a\x16.agent.GeneralResponse\"\x00\x12@\n\tCheckUser\x12\x17.agent.CheckUserRequest\x1a\x18.agent.CheckUserResponse\"\x00\x12I\n\x0cRegisterUser\x12\x1a.agent.RegisterUserRequest\x1a\x1b.agent.RegisterUserResponse\"\x00\x12:\n\tRunModule\x12\x17.agent.ModuleRunRequest\x1a\x10.agent.ModuleRun\"\x00\x30\x01\x12;\n\x0e\x43heckModuleRun\x12\x15.agent.ModuleRunCheck\x1a\x10.agent.ModuleRun\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc_server_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_GENERALRESPONSE']._serialized_start=87
  _globals['_GENERALRESPONSE']._serialized_end=133
  _globals['_LLMCONFIG']._serialized_start=136
  _globals['_LLMCONFIG']._serialized_end=369
  _globals['_NODESERVER']._serialized_start=371
  _globals['_NODESERVER']._serialized_end=435
  _globals['_NODECONFIGINPUT']._serialized_start=437
  _globals['_NODECONFIGINPUT']._serialized_end=546
  _globals['_NODECONFIG']._serialized_start=549
  _globals['_NODECONFIG']._serialized_end=1004
  _globals['_MODULE']._serialized_start=1007
  _globals['_MODULE']._serialized_end=1290
  _globals['_DATAGENERATIONCONFIG']._serialized_start=1293
  _globals['_DATAGENERATIONCONFIG']._serialized_end=1629
  _globals['_BASEDEPLOYMENT']._serialized_start=1632
  _globals['_BASEDEPLOYMENT']._serialized_end=1851
  _globals['_TOOLDEPLOYMENT']._serialized_start=1854
  _globals['_TOOLDEPLOYMENT']._serialized_end=2166
  _globals['_AGENTDEPLOYMENT']._serialized_start=2169
  _globals['_AGENTDEPLOYMENT']._serialized_end=2634
  _globals['_MODULERUNREQUEST']._serialized_start=2637
  _globals['_MODULERUNREQUEST']._serialized_end=3056
  _globals['_MODULERUN']._serialized_start=3059
  _globals['_MODULERUN']._serialized_end=3843
  _globals['_MODULERUNCHECK']._serialized_start=3845
  _globals['_MODULERUNCHECK']._serialized_end=3898
  _globals['_CHECKUSERREQUEST']._serialized_start=3900
  _globals['_CHECKUSERREQUEST']._serialized_end=3972
  _globals['_CHECKUSERRESPONSE']._serialized_start=3974
  _globals['_CHECKUSERRESPONSE']._serialized_end=4048
  _globals['_REGISTERUSERREQUEST']._serialized_start=4050
  _globals['_REGISTERUSERREQUEST']._serialized_end=4091
  _globals['_REGISTERUSERRESPONSE']._serialized_start=4093
  _globals['_REGISTERUSERRESPONSE']._serialized_end=4147
  _globals['_GRPCSERVER']._serialized_start=4150
  _globals['_GRPCSERVER']._serialized_end=4544
# @@protoc_insertion_point(module_scope)
